/* COPYRIGHT NOTICE
 *
 * This program comes from "The X Toolkit Cookbook" by Paul E. Kimball
 * c. 1995 Prentice Hall.
 *
 *  HDR1: Example 8-9 - counter.c - Work procedures
 *  
 *  HDR2: Chapter 8, page 209
 *
 * This software is provided free of charge and may be freely distributed
 * in its original form, so long as this notice appears unchanged in each 
 * copy.  This software is supplied "as-is" and is not covered by warrantee
 * of any kind, express or implied.  
 *
 * This and other examples are available by anonymous ftp from
 * ftp.netcom.com, directory "pub/pk/pkimball/cookbook".
 *
 *  1/15/95	Paul Kimball - pkimball@netcom.com
 */

/***  Include Required Header Files    *******************************/

#include <X11/Intrinsic.h>     /* The Intrinsics                     */
#include <Xm/Xm.h>             /* Motif Symbols                      */
#include <Xm/RowColumn.h>      /* Motif XmRowColumn                  */
#include <Xm/PushB.h>          /* Motif XmPushButton                 */
#include <Xm/Label.h>          /* Motif XmLabel                      */
#include <Xm/Text.h>           /* Motif XmText                       */

/***** Declare application data type *********************************/

 typedef struct {              /* client_data structure              */
   XtAppContext  context;      /* Application context                */
   XtIntervalId cyclic_timer;  /* ID of timer proc                   */
   XtWorkProcId counter_id;    /* ID of work proc                    */
   Widget       text;          /* Widget ID of text widget           */
   unsigned int count;         /* Current value of count             */
   unsigned int target;        /* Input number to count to           */
   Boolean      done;          /* True if count is finished          */
 } WorkData;                   /* structure type                     */

/***** Forward declare callback routines *****************************/

void ExitCallback();    /* Exits                                     */
void StartCounter();    /* Callback to start work procedure          */
void StopCounter();     /* Callback to stop work procedure           */
void ShowProgress();    /* Timer proc to check count status          */
Boolean Counter();      /* Work proc that counts by 1 each time      */

/***** Main Program  *************************************************/

main(argc,argv)
int argc;
char **argv;
{
 Widget  toplevel, manager, top_label,            /* Widget tree     */
         text_area, top_button, bottom_button;
         
 WorkData wdata;                  /* Allocate client_data struct     */
 wdata.done = True;               /* Initialize important fields     */

/*** Init. Toolkit, Appl. Context, Application Shell  ****************/

 toplevel = XtAppInitialize( &(wdata.context), "Counter", NULL, 0,
     &argc, argv, NULL, NULL, 0);

/***  Create widget tree, bind callbacks  ****************************/

 manager = XtCreateManagedWidget("manager",
     xmRowColumnWidgetClass, toplevel, NULL, 0);

 top_label = XtCreateManagedWidget("topLabel",
     xmLabelWidgetClass, manager, NULL, 0);

 text_area = XtCreateManagedWidget("textArea",
     xmTextWidgetClass, manager, NULL, 0);
 XtAddCallback( text_area, XmNactivateCallback,
     StartCounter, (XtPointer)&wdata );

 wdata.text = text_area;         /***    Initialize text ID    *******/

 top_button = XtCreateManagedWidget("topButton",
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( top_button, XmNactivateCallback,
     StopCounter, (XtPointer)&wdata );

 bottom_button = XtCreateManagedWidget("bottomButton",
     xmPushButtonWidgetClass, manager, NULL, 0);
 XtAddCallback( bottom_button, XmNactivateCallback, ExitCallback, NULL );

 XtRealizeWidget(toplevel);      /***    Realize all        **********/
 XtAppMainLoop(wdata.context);   /***    Start main loop    **********/
}

/*****  Callback Routines ********************************************/

void StartCounter(w, client_data, call_data)   /* starts counter     */
Widget  w;                                     /* unused             */
XtPointer client_data;                         /* WorkData pointer   */
XtPointer call_data;                           /* unused             */
{

 WorkData *wdp = (WorkData*)client_data;       /* cast client_data   */ 

 String buffer;                                /* Input buffer       */
 int    target;                                /* Input number       */

/*****   Fetch input   ***********************************************/

 XtVaGetValues(wdp->text, XmNvalue, &buffer, NULL );  

/*****   Scan input    ***********************************************/

 if( !sscanf(buffer, "%i", &target) ){                
    XtVaSetValues(w, XmNvalue, "Try again", NULL);     /* Failed     */
 }
 else {                                                /* Got number */

    wdp->count = 0;                            /* Initialize counter */
    wdp->target = target;                      /* Initialize target  */
    wdp->done = False;                         /* Initialize flag    */
   
/*****  Register work proc and cyclic timer to check progress  *******/

    wdp->counter_id = XtAppAddWorkProc(wdp->context, Counter, wdp);
    wdp->cyclic_timer = XtAppAddTimeOut(wdp->context, 1000, ShowProgress, wdp);

/*****  Tell user we're counting and prevent further input  **********/

    XtVaSetValues(wdp->text, XmNvalue, "Starting Count", NULL );
    XtVaSetValues(wdp->text, XmNeditable, False, NULL);
 }
}

/*********************************************************************/

Boolean Counter(client_data)                   /* Counting routine   */
XtPointer client_data;                         /* WorkData pointer   */
{

 WorkData* wdp = (WorkData*)client_data;       /* cast client_data   */

 wdp->count++;                                 /* increment counter  */

 if (wdp->count == wdp->target) {              /* counted enough?    */
   wdp->done = True;                           /* count is finished  */
   return True;              /* done; work proc will be removed      */
 }
 else return False;          /* not done; work proc will be requeued */

}

/***********************************************************************/

void ShowProgress(client_data, timer)            /* monitors the count */
XtPointer client_data;                           /* WorkData pointer   */
XtIntervalId *timer;                             /* unused             */
{

 WorkData *wdp = (WorkData*)client_data;         /* cast client_data   */ 

 char buffer[40];                                /* text message buffer*/

 if (wdp->done) {                                /* check "done" flag  */

/*****   count is finished - tell user and allow new input   ***********/

    sprintf(buffer, "Count stopped at %i", wdp->count);
    XtVaSetValues(wdp->text, XmNvalue, buffer, NULL);
    XtVaSetValues(wdp->text, XmNeditable, True, NULL);
 }
 else {

/*****   Still counting - update display and check again later   *******/

    sprintf(buffer, "%i", wdp->count);
    XtVaSetValues(wdp->text, XmNvalue, buffer, NULL ); 
    wdp->cyclic_timer = XtAppAddTimeOut(wdp->context, 1000, ShowProgress, wdp);
 }
}

/***********************************************************************/

void StopCounter(w, client_data, call_data)      /* halts the count    */
Widget  w;                                       /* unused             */
XtPointer client_data;                           /* WorkData pointer   */
XtPointer call_data;                             /* unused             */
{

 WorkData *wdp = (WorkData*)client_data;         /* cast client_data   */

 if(!wdp->done){                                 /* check "done" flag  */
   XtRemoveWorkProc(wdp->counter_id);            /* cancel work proc   */
   wdp->done = True;                             /* count is finished  */
 }

}

/***********************************************************************/

void ExitCallback(w, client_data, call_data)     /* exit routine       */
Widget  w;                                       /* unused             */
XtPointer client_data, call_data;                /* unused             */
{
  exit(1);
}

